#include "objpose.h"

#include <ArmarXCore/core/exceptions/local/UnexpectedEnumValueException.h>

#include <RobotAPI/libraries/aron/common/aron_conversions.h>
#include <RobotAPI/libraries/ArmarXObjects/aron_conversions.h>


void armarx::objpose::fromAron(const arondto::ObjectAttachmentInfo& dto, ObjectAttachmentInfo& bo)
{
    bo.frameName = dto.frameName;
    bo.agentName = dto.agentName;
    bo.poseInFrame = dto.poseInFrame;
}
void armarx::objpose::toAron(arondto::ObjectAttachmentInfo& dto, const ObjectAttachmentInfo& bo)
{
    dto.frameName = bo.frameName;
    dto.agentName = bo.agentName;
    dto.poseInFrame = bo.poseInFrame;
}

void armarx::objpose::fromAron(const arondto::ObjectType& dto, ObjectTypeEnum& bo)
{
    switch (dto.value)
    {
        case arondto::ObjectType::AnyObject:
            bo = ObjectTypeEnum::AnyObject;
            return;
        case arondto::ObjectType::KnownObject:
            bo = ObjectTypeEnum::KnownObject;
            return;
        case arondto::ObjectType::UnknownObject:
            bo = ObjectTypeEnum::UnknownObject;
            return;
    }
    ARMARX_UNEXPECTED_ENUM_VALUE(arondto::ObjectType, dto.value);
}
void armarx::objpose::toAron(arondto::ObjectType& dto, const ObjectTypeEnum& bo)
{
    switch (bo)
    {
        case ObjectTypeEnum::AnyObject:
            dto.value = arondto::ObjectType::AnyObject;
            return;
        case ObjectTypeEnum::KnownObject:
            dto.value = arondto::ObjectType::KnownObject;
            return;
        case ObjectTypeEnum::UnknownObject:
            dto.value = arondto::ObjectType::UnknownObject;
            return;
    }
    ARMARX_UNEXPECTED_ENUM_VALUE(ObjectTypeEnum, bo);
}


void armarx::objpose::fromAron(const arondto::ObjectPose& dto, ObjectPose& bo)
{
    bo.providerName = dto.providerName;

    fromAron(dto.objectType, bo.objectType);
    fromAron(dto.objectID, bo.objectID);

    bo.objectPoseRobot = dto.objectPoseRobot;
    bo.objectPoseGlobal = dto.objectPoseGlobal;
    bo.objectPoseOriginal = dto.objectPoseOriginal;
    bo.objectPoseOriginalFrame = dto.objectPoseOriginalFrame;

    bo.robotConfig = dto.robotConfig;
    bo.robotPose = dto.robotPose;

    if (dto.attachmentValid)
    {
        bo.attachment = ObjectAttachmentInfo();
        fromAron(dto.attachment, *bo.attachment);
    }
    else
    {
        bo.attachment = std::nullopt;
    }

    bo.confidence = dto.confidence;

    bo.timestamp = IceUtil::Time::microSeconds(dto.timestamp);

    if (dto.localOOBBValid)
    {
        bo.localOOBB = simox::OrientedBoxf();
        fromAron(dto.localOOBB, *bo.localOOBB);
    }
    else
    {
        bo.localOOBB = std::nullopt;
    }
}


void armarx::objpose::toAron(arondto::ObjectPose& dto, const ObjectPose& bo)
{
    dto.providerName = bo.providerName;

    toAron(dto.objectType, bo.objectType);
    toAron(dto.objectID, bo.objectID);

    dto.objectPoseRobot = bo.objectPoseRobot;
    dto.objectPoseGlobal = bo.objectPoseGlobal;
    dto.objectPoseOriginal = bo.objectPoseOriginal;
    dto.objectPoseOriginalFrame = bo.objectPoseOriginalFrame;

    dto.robotConfig = bo.robotConfig;
    dto.robotPose = bo.robotPose;

    if (bo.attachment)
    {
        dto.attachmentValid = true;
        toAron(dto.attachment, *bo.attachment);
    }
    else
    {
        dto.attachmentValid = false;
        toAron(dto.attachment, ObjectAttachmentInfo());
    }

    dto.confidence = bo.confidence;

    dto.timestamp = bo.timestamp.toMicroSeconds();

    if (bo.localOOBB)
    {
        dto.localOOBBValid = true;
        toAron(dto.localOOBB, *bo.localOOBB);
    }
    else
    {
        dto.localOOBBValid = false;
        toAron(dto.localOOBB, simox::OrientedBoxf());
    }
}

