#include "SimpleReaderBase.h"

#include "RobotAPI/libraries/armem/client/ComponentPlugin.h"

namespace armarx::armem::client::util
{
    SimpleReaderBase::SimpleReaderBase(ComponentPluginUser& memoryClient) :
        component(memoryClient)
    {
    }

    void SimpleReaderBase::registerPropertyDefinitions(
        armarx::PropertyDefinitionsPtr& def)
    {
        ARMARX_DEBUG << "Writer: registerPropertyDefinitions";

        const std::string prefix = propertyPrefix();

        def->optional(props.memoryName, prefix + "Memory");
        def->optional(props.coreSegmentName, prefix + "CoreSegment");
    }

    void SimpleReaderBase::connect()
    {
        // Wait for the memory to become available and add it as dependency.
        ARMARX_IMPORTANT << "Writer: Waiting for memory '" << props.memoryName
                         << "' ...";
        auto result = component.useMemory(props.memoryName);
        if (not result.success)
        {
            ARMARX_ERROR << result.errorMessage;
            return;
        }

        ARMARX_IMPORTANT << "SimpleReaderBase: Connected to memory '"
                         << props.memoryName;

        memoryReaderClient.setReadingMemory(result.proxy);
    }

    std::mutex& SimpleReaderBase::memoryReaderMutex()
    {
        return memoryMutex;
    }

    const armem::client::Reader& SimpleReaderBase::memoryReader() const
    {
        return memoryReaderClient;
    }

    const SimpleReaderBase::Properties& SimpleReaderBase::properties() const
    {
        return props;
    }
} // namespace armarx::armem::client::util