#include "SimpleWriterBase.h"

#include "RobotAPI/libraries/armem/client/ComponentPlugin.h"

namespace armarx::armem::client::util
{
    SimpleWriterBase::SimpleWriterBase(ComponentPluginUser& component) :
        component(component)
    {
    }

    void
    SimpleWriterBase::registerPropertyDefinitions(armarx::PropertyDefinitionsPtr& def)
    {
        ARMARX_DEBUG << "Writer: registerPropertyDefinitions";

        const std::string prefix = propertyPrefix();

        props = defaultProperties();

        def->optional(props.memoryName, prefix + "Memory");
        def->optional(props.coreSegmentName, prefix + "CoreSegment");

        def->required(props.providerName,
                      prefix + "Provider",
                      "Name of this provider");
    }

    void SimpleWriterBase::connect()
    {
        // Wait for the memory to become available and add it as dependency.
        ARMARX_IMPORTANT << "SimpleWriterBase: Waiting for memory '"
                         << props.memoryName << "' ...";
        auto result = component.useMemory(props.memoryName);
        if (not result.success)
        {
            ARMARX_ERROR << result.errorMessage;
            return;
        }

        ARMARX_IMPORTANT << "SimpleWriterBase: Connected to memory '"
                         << props.memoryName;

        memoryWriterClient.setWritingMemory(result.proxy);
        // memoryReader.setReadingMemory(result.proxy);
    }

    std::mutex& SimpleWriterBase::memoryWriterMutex()
    {
        return memoryMutex;
    }

    armem::client::Writer& SimpleWriterBase::memoryWriter()
    {
        return memoryWriterClient;
    }
    const SimpleWriterBase::Properties& SimpleWriterBase::properties() const
    {
        return props;
    }
} // namespace armarx::armem::client::util