/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Reister ( fabian dot reister at kit dot edu )
 * @date       2021
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

#include <string>
#include <optional>
#include <mutex>
#include <unordered_map>

#include <ArmarXCore/core/logging/Logging.h>
#include "ArmarXCore/core/application/properties/PropertyDefinitionContainer.h"

// #include "ArmarXGui/libraries/RemoteGui/Client/Widgets.h"

#include "RobotAPI/components/ArViz/Client/Client.h"

#include "RobotAPI/libraries/armem/core/MemoryID.h"
#include "RobotAPI/libraries/armem_objects/types.h"

#include <RobotAPI/libraries/armem_robot_state/types.h>

namespace armarx::armem
{
    namespace server
    {
        class MemoryToIceAdapter;
    }

    namespace wm
    {
        class CoreSegment;
    }
}  // namespace armarx::armem


namespace armarx::armem::server::robot_state::localization
{
    class Visu;

    class Segment : public armarx::Logging
    {
    public:
        Segment(server::MemoryToIceAdapter& iceMemory,
                std::mutex& memoryMutex);

        virtual ~Segment();

        void connect(viz::Client arviz);

        void defineProperties(armarx::PropertyDefinitionsPtr defs, const std::string& prefix = "");

        void init();

        std::unordered_map<std::string, Eigen::Affine3f> getRobotGlobalPoses(const armem::Time& timestamp) const;

        bool storeTransform(const armarx::armem::robot_state::Transform& transform);

    private:

        server::MemoryToIceAdapter& iceMemory;
        wm::CoreSegment* coreSegment = nullptr;
        std::mutex& memoryMutex;

        struct Properties
        {
            std::string coreSegment = "Localization";
            int64_t maxHistorySize = -1;
        };
        Properties p;

        // std::unique_ptr<Visu> visu;

    public:

        // struct RemoteGui
        // {
        //     armarx::RemoteGui::Client::GroupBox group;

        //     armarx::RemoteGui::Client::IntSpinBox maxHistorySize;
        //     armarx::RemoteGui::Client::CheckBox infiniteHistory;
        //     armarx::RemoteGui::Client::CheckBox discardSnapshotsWhileAttached;

        //     void setup(const Segment& data);
        //     void update(Segment& data);
        // };

    };

}  // namespace armarx::armem::server::robot_state::localization
