/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include <RobotAPI/libraries/aron/core/navigator/type/Navigator.h>

namespace armarx::aron::typenavigator
{
    class IVTCByteImageNavigator;
    typedef std::shared_ptr<IVTCByteImageNavigator> IVTCByteImageNavigatorPtr;

    class IVTCByteImageNavigator :
        virtual public Navigator
    {
    public:
        using PointerType = IVTCByteImageNavigatorPtr;

    public:
        // constructors
        IVTCByteImageNavigator() = delete;
        IVTCByteImageNavigator(const Path& path);
        IVTCByteImageNavigator(const type::AronIVTCByteImagePtr&, const Path& path);

        std::string checkTypename(const std::string&) const;

        unsigned int getWidth() const;
        unsigned int getHeight() const;
        std::string getTypename() const;
        std::vector<int> getDimensions() const;

        void setWidth(const unsigned int&);
        void setHeight(const unsigned int&);
        void setTypename(const std::string&);

        type::AronIVTCByteImagePtr toAronIVTCByteImagePtr() const;

        // static methods
        static IVTCByteImageNavigatorPtr DynamicCast(const NavigatorPtr& n);
        static IVTCByteImageNavigatorPtr DynamicCastAndCheck(const NavigatorPtr& n);

        // virtual implementations
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;
        virtual type::AronTypePtr getResult() const override;
        virtual std::string getName() const override;

    public:
        const std::map<std::string, std::vector<std::string>> ACCEPTED_TYPES =
        {
            {"GrayScale", {"CByteImage::ImageType::eGrayScale", "ImageType::eGrayScale"}},
            {"RGB24", {"CByteImage::ImageType::eRGB24", "ImageType::eRGB24"}},
            {"RGB24Split", {"CByteImage::ImageType::eRGB24Split", "ImageType::eRGB24Split"}}
        };

    private:
        // members
        type::AronIVTCByteImagePtr type;
    };
}
