/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include <RobotAPI/libraries/aron/core/navigator/type/Navigator.h>

namespace armarx::aron::typenavigator
{
    class OpenCVMatNavigator;
    typedef std::shared_ptr<OpenCVMatNavigator> OpenCVMatNavigatorPtr;

    class OpenCVMatNavigator :
        virtual public Navigator
    {
    public:
        using PointerType = OpenCVMatNavigatorPtr;

    public:
        // constructors
        OpenCVMatNavigator() = delete;
        OpenCVMatNavigator(const Path& path);
        OpenCVMatNavigator(const type::AronOpenCVMatPtr&, const Path& path);

        bool checkDimensions(const std::vector<int>&) const;
        std::string checkTypename(const std::string&) const;

        std::string getTypename() const;
        std::vector<int> getDimensions() const;

        void setDimensions(const std::vector<int>&);
        void setTypename(const std::string&);

        type::AronOpenCVMatPtr toAronOpenCVMatPtr() const;

        // static methods
        static OpenCVMatNavigatorPtr DynamicCast(const NavigatorPtr& n);
        static OpenCVMatNavigatorPtr DynamicCastAndCheck(const NavigatorPtr& n);

        // virtual implementations
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;
        virtual type::AronTypePtr getResult() const override;
        virtual std::string getName() const override;

    public:
        const std::map<std::string, std::vector<std::string>> ACCEPTED_TYPES =
        {
            {"CV_8U", {"8U"}},
            {"CV_8S", {"8S"}},
            {"CV_16U", {"16U"}},
            {"CV_16S", {"16S"}},
            {"CV_32S", {"32S"}},
            {"CV_32F", {"32F"}},
            {"CV_64F", {"64F"}}
        };
        const unsigned int ACCEPTED_DIMENSION_MIN_SIZE = 2;

    private:
        // members
        type::AronOpenCVMatPtr type;
    };
}
